library teamdrt_firebase_package;

import 'dart:async';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:teamdrt_firebase_package/PackageConsts.dart';

/// Login Methods for Firebase
/// Make sure to initialise FirebaseApp before calling these methods
/// These Services will help you help you implement Login and SignUp methods for firebase

class SignInBloc {
  final FirebaseAuth _auth = FirebaseAuth.instance;

  Stream<User> get user {
    return _auth.authStateChanges();
  }

  var errorController = StreamController<String>.broadcast();
  var signInStatus = StreamController<SignInStatus>.broadcast();

  Future logInWithEmail([String email, String password]) async {
    try {
      signInStatus.sink.add(SignInStatus.PROCESSING);
      _auth.signInWithEmailAndPassword(email: email, password: password);
    } on FirebaseException catch (e) {
      failed(e.code);
    }
  }

  Future signOut() async {
    _auth.signOut();
  }

  Future loginWithGoogle() async {
    try {
      final GoogleSignIn googleSignIn = GoogleSignIn();
      final GoogleSignInAccount googleSignInAccount =
          await googleSignIn.signIn();
      final GoogleSignInAuthentication googleSignInAuthentication =
          await googleSignInAccount.authentication;
      print(googleSignInAuthentication.accessToken);
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
      _auth.signInWithCredential(credential);
    } on PlatformException catch (e) {
      failed(e.code);
    } on FirebaseException catch (e) {
      failed(e.code);
    } catch (e) {
      debugPrint(
        e,
      );
      failed(Constants.unknownError);
    }
  }

  failed(String error) {
    print(error);
    errorController.sink.add(error);
    signInStatus.sink.add(SignInStatus.FAILED);
  }

  dispose() {
    errorController.close();
    signInStatus.close();
  }
}

class SignUpBloc {}

enum SignInStatus {
  PROCESSING,
  EMAILSENT,
  COMPLETED,
  OTPSENT,
  FAILED,
}
